@@ -36,9 +36,7 @@
 #include <gtk/gtkenums.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 
 /* Binding sets
@@ -76,14 +74,6 @@ struct _GtkBindingEntry
   GtkBindingSignal	*signals;
 };
 
-struct _GtkBindingSignal
-{
-  GtkBindingSignal	*next;
-  gchar			*signal_name;
-  guint			 n_args;
-  GtkBindingArg		*args;
-};
-
 struct _GtkBindingArg
 {
   GType		 arg_type;
@@ -94,6 +84,13 @@ struct _GtkBindingArg
   } d;
 };
 
+struct _GtkBindingSignal
+{
+  GtkBindingSignal	*next;
+  gchar 		*signal_name;
+  guint			 n_args;
+  GtkBindingArg		*args;
+};
 
 /* Application-level methods */
 
@@ -103,6 +100,8 @@ GtkBindingSet*	gtk_binding_set_find	(const gchar	*set_name);
 gboolean gtk_bindings_activate		(GtkObject	*object,
 					 guint		 keyval,
 					 GdkModifierType modifiers);
+gboolean gtk_bindings_activate_event    (GtkObject      *object,
+					 GdkEventKey    *event);
 gboolean gtk_binding_set_activate	(GtkBindingSet	*binding_set,
 					 guint		 keyval,
 					 GdkModifierType modifiers,
@@ -136,8 +135,6 @@ void	 gtk_binding_entry_add_signall	(GtkBindingSet	*binding_set,
 guint	 gtk_binding_parse_binding	(GScanner	*scanner);
 
 
-gboolean _gtk_bindings_activate_event (GtkObject   *object,
-				       GdkEventKey *event);
 void     _gtk_binding_reset_parsed    (void);
 
 /* Creates a signal with a fixed callback instead of a class offset;
@@ -153,9 +150,8 @@ guint _gtk_binding_signal_new (const gchar       *signal_name,
 			       GType		  return_type,
 			       guint		  n_params,
 			       ...);
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+
+G_END_DECLS
 
 
 #endif /* __GTK_BINDINGS_H__ */
