@@ -1,3 +1,4 @@
+#include "gtkmarshal.h"
 #include "gtkalias.h"
 
 #include	<glib-object.h>
@@ -1112,3 +1113,5 @@ gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
             data2);
 }
 
+#define __gtk_marshal_MARSHAL_C__
+#include "gtkaliasdef.c"
