@@ -42,6 +42,7 @@
 #include "gtkseparatormenuitem.h"
 #include "gtkmenu.h"
 #include "gtkimage.h"
+#include "gtklabel.h"
 #include "gtkobject.h"
 #include "gtktooltips.h"
 #include "gtktypebuiltins.h"
@@ -62,6 +63,9 @@ struct _GtkRecentChooserMenuPrivate
   /* size of the icons of the menu items */  
   gint icon_size;
 
+  /* max size of the menu item label */
+  gint label_width;
+
   /* RecentChooser properties */
   gint limit;  
   guint show_private : 1;
@@ -97,10 +101,12 @@ enum {
 
 #define FALLBACK_ICON_SIZE 	32
 #define FALLBACK_ITEM_LIMIT 	10
+#define DEFAULT_LABEL_WIDTH     30
 
 #define GTK_RECENT_CHOOSER_MENU_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_RECENT_CHOOSER_MENU, GtkRecentChooserMenuPrivate))
 
 static void     gtk_recent_chooser_menu_finalize    (GObject                   *object);
+static void     gtk_recent_chooser_menu_dispose     (GObject                   *object);
 static GObject *gtk_recent_chooser_menu_constructor (GType                      type,
 						     guint                      n_construct_properties,
 						     GObjectConstructParam     *construct_params);
@@ -189,6 +195,7 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
 
   gobject_class->constructor = gtk_recent_chooser_menu_constructor;
   gobject_class->finalize = gtk_recent_chooser_menu_finalize;
+  gobject_class->dispose = gtk_recent_chooser_menu_dispose;
   gobject_class->set_property = gtk_recent_chooser_menu_set_property;
   gobject_class->get_property = gtk_recent_chooser_menu_get_property;
 
@@ -234,6 +241,8 @@ gtk_recent_chooser_menu_init (GtkRecentChooserMenu *menu)
   priv->sort_type = GTK_RECENT_SORT_NONE;
   
   priv->icon_size = FALLBACK_ICON_SIZE;
+
+  priv->label_width = DEFAULT_LABEL_WIDTH;
   
   priv->current_filter = NULL;
     
@@ -247,32 +256,55 @@ gtk_recent_chooser_menu_finalize (GObject *object)
   GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (object);
   GtkRecentChooserMenuPrivate *priv = menu->priv;
   
-  g_signal_handler_disconnect (priv->manager, priv->manager_changed_id);
-  priv->manager_changed_id = 0;
-
-  if (priv->populate_id)
-    g_source_remove (priv->populate_id);
-
   priv->manager = NULL;
   
   if (priv->sort_data_destroy)
     {
       priv->sort_data_destroy (priv->sort_data);
-      
       priv->sort_data_destroy = NULL;
-      priv->sort_data = NULL;
-      priv->sort_func = NULL;
     }
   
-  if (priv->tooltips)
-    g_object_unref (priv->tooltips);
-  
-  if (priv->current_filter)
-    g_object_unref (priv->current_filter);
+  priv->sort_data = NULL;
+  priv->sort_func = NULL;
   
   G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->finalize (object);
 }
 
+static void
+gtk_recent_chooser_menu_dispose (GObject *object)
+{
+  GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (object);
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
+
+  if (priv->manager_changed_id)
+    {
+      if (priv->manager)
+        g_signal_handler_disconnect (priv->manager, priv->manager_changed_id);
+
+      priv->manager_changed_id = 0;
+    }
+
+  if (priv->populate_id)
+    {
+      g_source_remove (priv->populate_id);
+      priv->populate_id = 0;
+    }
+
+  if (priv->tooltips)
+    {
+      g_object_unref (priv->tooltips);
+      priv->tooltips = NULL;
+    }
+
+  if (priv->current_filter)
+    {
+      g_object_unref (priv->current_filter);
+      priv->current_filter = NULL;
+    }
+
+  G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->dispose (object);
+}
+
 static GObject *
 gtk_recent_chooser_menu_constructor (GType                  type,
 				     guint                  n_construct_properties,
@@ -411,26 +443,28 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
       if (!info)
         continue;
       
-      if (0 == strcmp (uri, gtk_recent_info_get_uri (info)))
-        found = TRUE;
+      if (strcmp (uri, gtk_recent_info_get_uri (info)) == 0)
+        {
+          gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu),
+	                                menu_item,
+					TRUE);
+          found = TRUE;
+
+	  break;
+	}
     }
 
   g_list_free (children);
   
-  if (!found)  
+  if (!found)
     {
       g_set_error (error, GTK_RECENT_CHOOSER_ERROR,
       		   GTK_RECENT_CHOOSER_ERROR_NOT_FOUND,
       		   _("No recently used resource found with URI `%s'"),
       		   uri);
-      return FALSE;
-    }
-  else
-    {
-      gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu), menu_item, TRUE);
-      
-      return TRUE;
     }
+
+  return found;
 }
 
 static gchar *
@@ -612,8 +646,10 @@ gtk_recent_chooser_menu_get_items (GtkRecentChooser *chooser)
     return NULL;
   
   limit = gtk_recent_chooser_get_limit (chooser);
-  sort_type = gtk_recent_chooser_get_sort_type (chooser);
+  if (limit == 0)
+    return NULL;
 
+  sort_type = gtk_recent_chooser_get_sort_type (chooser);
   switch (sort_type)
     {
     case GTK_RECENT_SORT_NONE:
@@ -632,7 +668,7 @@ gtk_recent_chooser_menu_get_items (GtkRecentChooser *chooser)
       g_assert_not_reached ();
       break;
     }
-  
+
   items = gtk_recent_manager_get_items (priv->manager);
   if (!items)
     return NULL;
@@ -646,6 +682,8 @@ gtk_recent_chooser_menu_get_items (GtkRecentChooser *chooser)
       GList *clamp, *l;
 
       clamp = g_list_nth (items, limit - 1);
+      if (!clamp)
+        return items;
 
       l = clamp->next;
       clamp->next = NULL;      
@@ -848,16 +886,18 @@ gtk_recent_chooser_menu_add_tip (GtkRecentChooserMenu *menu,
     return;
   
   path = gtk_recent_info_get_uri_display (info);
-  
-  tip_text = g_strdup_printf (_("Open '%s'"), path);
- 
-  gtk_tooltips_set_tip (priv->tooltips,
-  			item,
-  			tip_text,
-  			NULL);
+  if (path)
+    {
+      tip_text = g_strdup_printf (_("Open '%s'"), path);
 
-  g_free (path);  
-  g_free (tip_text);
+      gtk_tooltips_set_tip (priv->tooltips,
+                            item,
+                            tip_text,
+                            NULL);
+      
+      g_free (path);
+      g_free (tip_text);
+    }
 }
 
 static GtkWidget *
@@ -866,8 +906,8 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
 				     gint                  count)
 {
   GtkRecentChooserMenuPrivate *priv;
-  gchar *label;
-  GtkWidget *item, *image;
+  gchar *text;
+  GtkWidget *item, *image, *label;
   GdkPixbuf *icon;
 
   g_assert (info != NULL);
@@ -886,19 +926,28 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
       
       /* avoid clashing mnemonics */
       if (count <= 10)
-        label = g_strdup_printf ("_%d. %s", count, escaped);
+        text = g_strdup_printf ("_%d. %s", count, escaped);
       else
-        label = g_strdup_printf ("%d. %s", count, escaped);
+        text = g_strdup_printf ("%d. %s", count, escaped);
       
-      item = gtk_image_menu_item_new_with_mnemonic (label);
+      item = gtk_image_menu_item_new_with_mnemonic (text);
       
       g_free (escaped);
       g_free (name);
     }
   else
     {
-      label = g_strdup (gtk_recent_info_get_display_name (info));
-      item = gtk_image_menu_item_new_with_label (label);
+      text = g_strdup (gtk_recent_info_get_display_name (info));
+      item = gtk_image_menu_item_new_with_label (text);
+    }
+
+  g_free (text);
+
+  label = GTK_BIN (item)->child;
+  if (label)
+    {
+      gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+      gtk_label_set_max_width_chars (GTK_LABEL (label), priv->label_width);
     }
   
   if (priv->show_icons)
@@ -907,15 +956,14 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
         
       image = gtk_image_new_from_pixbuf (icon);
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+      g_object_unref (icon);
     }
 
   g_signal_connect (item, "activate",
   		    G_CALLBACK (item_activate_cb),
   		    menu);
 
-out:
-  g_free (label);
-
   return item;
 }
 
@@ -1138,21 +1186,25 @@ static void
 set_recent_manager (GtkRecentChooserMenu *menu,
 		    GtkRecentManager     *manager)
 {
-  if (menu->priv->manager)
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
+
+  if (priv->manager)
     {
-      g_signal_handler_disconnect (menu, menu->priv->manager_changed_id);
-      menu->priv->manager = NULL;
+      if (priv->manager_changed_id)
+        g_signal_handler_disconnect (priv->manager, priv->manager_changed_id);
+
+      priv->manager = NULL;
     }
   
   if (manager)
-    menu->priv->manager = manager;
+    priv->manager = manager;
   else
-    menu->priv->manager = gtk_recent_manager_get_default ();
+    priv->manager = gtk_recent_manager_get_default ();
   
-  if (menu->priv->manager)
-    menu->priv->manager_changed_id = g_signal_connect (menu->priv->manager, "changed",
-      						       G_CALLBACK (manager_changed_cb),
-      						       menu);
+  if (priv->manager)
+    priv->manager_changed_id = g_signal_connect (priv->manager, "changed",
+      						 G_CALLBACK (manager_changed_cb),
+      						 menu);
   /* (re)populate the menu */
   gtk_recent_chooser_menu_populate (menu);
 }
